﻿using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Text;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 루트 노드
        /// </summary>
        private TreeNode<ImageNode> rootNode = new TreeNode<ImageNode>
        (
            new ImageNode
            (
                "Queen Elizabeth II && Prince Philip Duke of Edinburgh",
                Properties.Resources.Elizabeth_Philip
            )
        );

        /// <summary>
        /// 선택 노드
        /// </summary>
        private TreeNode<ImageNode> selectedNode;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            #region 이벤트를 설정한다.

            Load                             += Form_Load;
            this.canvasPictureBox.MouseClick += canvasPictureBox_MouseClick;

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 폼 로드시 처리하기 - Form_Load(sender, e)

        /// <summary>
        /// 폼 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Form_Load(object sender, EventArgs e)
        {
            TreeNode<ImageNode> charlesNode = new TreeNode<ImageNode>
            (
                new ImageNode
                (
                    "Prince Charles, Prince of Wales",
                    Properties.Resources.Charles
                )
            );

            TreeNode<ImageNode> anneNode = new TreeNode<ImageNode>
            (
                new ImageNode
                (
                    "Princess Anne",
                    Properties.Resources.Anne
                )
            );

            TreeNode<ImageNode> andrewNode = new TreeNode<ImageNode>
            (
                new ImageNode
                (
                    "Prince Andrew, Duke of York",
                    Properties.Resources.Andrew
                )
            );

            TreeNode<ImageNode> edwardNode = new TreeNode<ImageNode>
            (
                new ImageNode
                (
                    "Prince Edward, Earl of Wessex",
                    Properties.Resources.Edward
                )
            );

            TreeNode<ImageNode> williamNode = new TreeNode<ImageNode>
            (
                new ImageNode
                (
                    "Prince William",
                    Properties.Resources.William
                )
            );

            TreeNode<ImageNode> harryNode = new TreeNode<ImageNode>
            (
                new ImageNode
                (
                    "Prince Henry (Harry)",
                    Properties.Resources.Harry
                )
            );

            TreeNode<ImageNode> peterNode = new TreeNode<ImageNode>
            (
                new ImageNode
                (
                    "Peter Phillips",
                    Properties.Resources.Peter
                )
            );

            TreeNode<ImageNode> zaraNode = new TreeNode<ImageNode>
            (
                new ImageNode
                (
                    "Zara Phillips",
                    Properties.Resources.Zara
                )
            );

            TreeNode<ImageNode> beatriceNode = new TreeNode<ImageNode>
            (
                new ImageNode
                (
                    "Princess Beatrice",
                    Properties.Resources.Beatrice
                )
            );

            TreeNode<ImageNode> eugenieNode = new TreeNode<ImageNode>
            (
                new ImageNode
                (
                    "Princess Eugenie",
                    Properties.Resources.Eugenie
                )
            );

            TreeNode<ImageNode> louiseNode = new TreeNode<ImageNode>
            (
                new ImageNode
                (
                    "Lady Louise",
                    Properties.Resources.Louise
                )
            );

            TreeNode<ImageNode> severnNode = new TreeNode<ImageNode>
            (
                new ImageNode
                (
                    "Viscount Severn",
                    Properties.Resources.Severn
                )
            );

            this.rootNode.AddChild(charlesNode);

            charlesNode.AddChild(williamNode);
            charlesNode.AddChild(harryNode);

            this.rootNode.AddChild(anneNode);

            anneNode.AddChild(peterNode);
            anneNode.AddChild(zaraNode);

            this.rootNode.AddChild(andrewNode);

            andrewNode.AddChild(beatriceNode);
            andrewNode.AddChild(eugenieNode);

            this.rootNode.AddChild(edwardNode);

            edwardNode.AddChild(louiseNode);
            edwardNode.AddChild(severnNode);


            this.contentPanel.AutoScroll = true;

            this.canvasPictureBox.Location = new Point(0, 0);
            this.canvasPictureBox.SizeMode = PictureBoxSizeMode.AutoSize;
            this.canvasPictureBox.Image    = DrawTree(rootNode, 5);
        }

        #endregion
        #region 캔버스 픽처 박스  마우스 클릭시 처리하기 - canvasPictureBox_MouseClick(sender, e)

        /// <summary>
        /// 캔버스 픽처 박스  마우스 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void canvasPictureBox_MouseClick(object sender, MouseEventArgs e)
        {
            FindNodeUnderMouse(e.Location);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 트리 그리기 - DrawTree(rootNode, margin)

        /// <summary>
        /// 트리 그리기
        /// </summary>
        /// <param name="rootNode">루트 노드</param>
        /// <param name="margin">마진</param>
        /// <returns>비트맵</returns>
        private Bitmap DrawTree(TreeNode<ImageNode> rootNode, int margin)
        {
            float minimumX = margin;
            float minimumY = margin;

            using(Bitmap sourceBitmap = new Bitmap(10, 10))
            {
                using(Graphics sourceGraphics = Graphics.FromImage(sourceBitmap))
                {
                    rootNode.Arrange(sourceGraphics, ref minimumX, ref minimumY);
                }
            }

            int width  = (int)minimumX + margin;
            int height = (int)minimumY + margin;

            Bitmap targetBitmap = new Bitmap(width, height);

            using(Graphics tragetBitmap = Graphics.FromImage(targetBitmap))
            {
                tragetBitmap.SmoothingMode     = SmoothingMode.AntiAlias;
                tragetBitmap.TextRenderingHint = TextRenderingHint.AntiAliasGridFit;

                rootNode.DrawTree(tragetBitmap);
            }

            return targetBitmap;
        }

        #endregion
        #region 마우스 아래 노드 찾기 - FindNodeUnderMouse(point)

        /// <summary>
        /// 마우스 아래 노드 찾기
        /// </summary>
        /// <param name="point">포인트</param>
        private void FindNodeUnderMouse(PointF point)
        {
            if(selectedNode != null)
            {
                this.selectedNode.Data.Selected = false;

                this.statusLabel.Text = string.Empty;
            }

            using(Graphics graphics = this.canvasPictureBox.CreateGraphics())
            {
                this.selectedNode = this.rootNode.GetNode(graphics, point);
            }

            if(this.selectedNode != null)
            {
                this.selectedNode.Data.Selected = true;

                this.statusLabel.Text = this.selectedNode.Data.Description;
            }

            this.canvasPictureBox.Refresh();
        }

        #endregion
    }
}